package com.bugsee;

import android.os.Handler;
import android.text.TextUtils;
import android.view.View;
import android.app.Activity;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.Rect;

import com.bugsee.library.Bugsee;
import com.bugsee.library.BugseeInternalAdapter;
import com.bugsee.library.ExceptionLoggingOptions;
import com.bugsee.library.data.IssueSeverity;
import com.bugsee.library.data.BugseeAppearance;
import com.bugsee.library.data.VideoQuality;
import com.bugsee.library.data.VideoMode;
import com.bugsee.library.exchange.ExchangeNetworkEvent;
import com.bugsee.library.events.BugseeLogLevel;
import com.bugsee.library.feedback.OnNewFeedbackListener;
import com.bugsee.library.logs.BugseeLog;
import com.bugsee.library.logs.LogFilter;
import com.bugsee.library.logs.LogListener;
import com.bugsee.library.network.NetworkEventFilter;
import com.bugsee.library.network.NetworkEventListener;
import com.bugsee.library.network.data.BugseeNetworkEvent;
import com.bugsee.library.attachment.CustomAttachment;
import com.bugsee.library.attachment.ReportAttachmentsProvider;
import com.bugsee.library.attachment.Report;
import com.bugsee.library.lifecycle.LifecycleEventListener;
import com.bugsee.library.lifecycle.LifecycleEventTypes;

import com.bugsee.library.network.data.BugseeNetworkEventImpl;
import com.bugsee.library.serverapi.data.event.NetworkEvent;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableMapKeySetIterator;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.facebook.react.modules.network.OkHttpClientFactory;
import com.facebook.react.modules.network.ReactCookieJarContainer;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import okhttp3.OkHttpClient;

public class BugseeNativeBridge extends ReactContextBaseJavaModule {
    private static class DefaultNetworkInterceptorFactory implements OkHttpClientFactory {
        public OkHttpClient createNewNetworkModuleClient() {
            return Bugsee.addNetworkLoggingToOkHttpBuilder(new OkHttpClient.Builder())
                    .cookieJar(new ReactCookieJarContainer())
                    .build();
        }
    }

    private static class RNUtils {
        private static boolean mapHasKey(ReadableMap params, String key) {
            return params != null && params.hasKey(key);
        }

        public static String optString(ReadableMap params, String key, String defaultValue) {
            String value = defaultValue;
            if (mapHasKey(params, key) && params.getType(key) == ReadableType.String) {
                value = params.getString(key);
            }
            return value;
        }

        public static boolean optBoolean(ReadableMap params, String key, boolean defaultValue) {
            boolean value = defaultValue;
            if (mapHasKey(params, key) && params.getType(key) == ReadableType.Boolean) {
                value = params.getBoolean(key);
            }
            return value;
        }

        public static int optInt(ReadableMap params, String key, int defaultValue) {
            int value = defaultValue;
            if (mapHasKey(params, key) && params.getType(key) == ReadableType.Number) {
                value = params.getInt(key);
            }
            return value;
        }

        public static double optDouble(ReadableMap params, String key, double defaultValue) {
            double value = defaultValue;
            if (mapHasKey(params, key) && params.getType(key) == ReadableType.Number) {
                value = params.getDouble(key);
            }
            return value;
        }

        public static ReadableMap optReadableMap(ReadableMap params, String key, ReadableMap defaultValue) {
            ReadableMap value = defaultValue;
            if (mapHasKey(params, key) && params.getType(key) == ReadableType.Map) {
                value = params.getMap(key);
            }
            return value;
        }

        public static int colorFromString(String colorString) {
            int color = 0;
            try {
                color = Color.parseColor(colorString);
            } catch (Exception e) {
                // e.printStackTrace();
            }
            return color;
        }
    }

    private final Object attachmentSyncObj = new Object();
    private final ReactApplicationContext reactContext;
    private final String packageName;
    private final HashMap<String, Boolean> flags;

    private final String BGS_NETFLT_EVENT_NAME = "bgsNetFilterEvent";
    private final String BGS_LOGLFT_EVENT_NAME = "bgsLogFilterEvent";
    private final String BGS_ATTACHMENTS_EVENT_NAME = "bgsAttachmentsEvent";
    private final String BGS_NEW_FEEDBACK_MESSAGES_EVENT_NAME = "bgsNewFeedbackMessagesEvent";
    private final String BGS_LIFECYCLE_EVENT_NAME = "bgsLifecycleEvent";

    private static class ReactNativeWebException extends Throwable {
        public ReactNativeWebException(String message) {
            super(message);
        }
    }

    public BugseeNativeBridge(ReactApplicationContext reactContext) {
        super(reactContext);

        this.flags = new HashMap<>();
        this.reactContext = reactContext;
        this.packageName = this.reactContext.getPackageName();

        this.setupEventsFilters();
    }

    private static Object getSpecialElementFromMap(ReadableMap map, String key) {
        Object object = null;
        switch (key) {
            case "VideoMode":
            case "VideoQuality":
            case "FallbackVideoMode":
            case "FrameRate":
            case "BugseeDefaultBugPriority":
            case "BugseeDefaultCrashPriority":
                object = Integer.valueOf(map.getInt(key));
                break;
        }
        return object;
    }

    private static Object getElementFromMap(ReadableMap map, String key, boolean handleSpecialElements) {
        ReadableType type = map.getType(key);
        Object object = handleSpecialElements ? getSpecialElementFromMap(map, key) : null;
        if (object == null) {
            switch(type) {
                case Number:
                    object = map.getDouble(key);
                    break;
                case String:
                    object = map.getString(key);
                    break;
                case Boolean:
                    object = map.getBoolean(key);
                    break;
                case Map:
                    object = toMap(map.getMap(key), handleSpecialElements);
                    break;
                case Array:
                    object = toList(map.getArray(key));
                    break;
            }
        }

        return object;
    }

    private static Object getElementFromArray(ReadableArray array, int index) {
        ReadableType type = array.getType(index);
        Object object = null;
        switch(type) {
            case Number:
                object = array.getDouble(index);
                break;
            case String:
                object = array.getString(index);
                break;
            case Boolean:
                object = array.getBoolean(index);
                break;
            case Map:
                object = toMap(array.getMap(index), false);
                break;
            case Array:
                object = toList(array.getArray(index));
                break;
        }

        return object;
    }

    private static ArrayList<Object> toList(ReadableArray array) {
        int size = array != null ? array.size() : 0;
        ArrayList<Object> arrayList = new ArrayList(size);

        if (size > 0) {
            for (int i = 0; i < array.size(); i++) {
                arrayList.add(getElementFromArray(array, i));
            }
        }

        return arrayList;
    }

    private static ArrayList<String> toStringList(ReadableArray array) {
        int size = array != null ? array.size() : 0;
        ArrayList<String> arrayList = new ArrayList<>(size);

        if (size > 0) {
            for (int i = 0; i < array.size(); i++) {
                Object objValue = getElementFromArray(array, i);
                arrayList.add(objValue != null ? objValue.toString() : null);
            }
        }

        return arrayList;
    }

    private static HashMap<String, Object> toMap(ReadableMap object, boolean handleSpecialElements) {
        HashMap<String, Object> map = new HashMap();
        if (object == null)
            return map;
        ReadableMapKeySetIterator keys = object.keySetIterator();
        while (keys.hasNextKey()) {
            String key = (String)keys.nextKey();
            map.put(key, getElementFromMap(object, key, handleSpecialElements));
        }
        return map;
    }

    private static boolean setClassPropertyValue(Object object, String fieldName, Object fieldValue) {
        Class<?> clazz = object.getClass();
        if (clazz != null) {
            try {
                Field field = clazz.getDeclaredField(fieldName);
                field.setAccessible(true);
                field.set(object, fieldValue);
                return true;
            } catch (Exception ignored) {
                // TODO: Remove this after testing
                // throw ignored;
            }
        }
        return false;
    }

    private void setupEventsFilters() {
        final HashMap<String, Boolean> flags = this.flags;

        Bugsee.setNetworkEventFilter(new NetworkEventFilter() {
            @Override
            public void filter(BugseeNetworkEvent bugseeNetworkEvent, NetworkEventListener networkEventListener) {
                if (flags.containsKey("netFilter") && flags.get("netFilter")) {
                    processNetworkEvent(bugseeNetworkEvent);
                    return;
                }

                networkEventListener.onEvent(bugseeNetworkEvent);
            }
        });

        Bugsee.setLogFilter(new LogFilter() {
            @Override
            public void filter(BugseeLog bugseeLog, LogListener logListener) {
                if (flags.containsKey("logFilter") && flags.get("logFilter")) {
                    processLogEvent(bugseeLog);
                    return;
                }

                logListener.onLog(bugseeLog);
            }
        });

        BugseeInternalAdapter.setAttachmentsAsync(true);

        final BugseeNativeBridge self = this;

        Bugsee.setReportAttachmentsProvider(new ReportAttachmentsProvider() {
            @Override
            public ArrayList<CustomAttachment> getAttachments(Report report) {
//                if (!self.flags.containsKey("")) {
//                    return new ArrayList<>();
//                }

                WritableMap reportData = Arguments.createMap();
                reportData.putInt("severity", report.getSeverity().getIntValue());
                reportData.putString("type", report.getType().toString());

                WritableMap eventData = Arguments.createMap();
                eventData.putMap("report", reportData);
                eventData.putString("id", UUID.randomUUID().toString());

                self.notifyClient(BGS_ATTACHMENTS_EVENT_NAME, eventData);

                return new ArrayList<>();
            }
        });

        Bugsee.setOnNewFeedbackListener(new OnNewFeedbackListener() {
            @Override
            public void onNewFeedback(List<String> list) {
                if (!self.flags.containsKey("feedbackHandler") || !self.flags.get("feedbackHandler")) {
                    return;
                }

                WritableMap eventData = Arguments.createMap();
                WritableArray messagesArray = Arguments.createArray();

                int count = list.size();
                for (int i = 0; i < count; ++i) {
                    messagesArray.pushString(list.get(i));
                }

                eventData.putArray("messages", messagesArray);

                self.notifyClient(BGS_NEW_FEEDBACK_MESSAGES_EVENT_NAME, eventData);
            }
        });

        Bugsee.setLifecycleEventsListener(new LifecycleEventListener() {
            @Override
            public void onEvent(LifecycleEventTypes eventType) {
                if (!self.flags.containsKey("lifecycleHandler") || !self.flags.get("lifecycleHandler")) {
                    return;
                }

                self.notifyClient(BGS_LIFECYCLE_EVENT_NAME, eventType.getIntValue());
            }
        });
    }

    private void processNetworkEvent(BugseeNetworkEvent bugseeNetworkEvent) {
        WritableMap netEvent = Arguments.createMap();

        BugseeNetworkEventImpl netEventImpl = (BugseeNetworkEventImpl)bugseeNetworkEvent;
        // We need all the details here
        NetworkEvent actualNetEvent = netEventImpl.getEvent();

        netEvent.putString("id", actualNetEvent.id);
        netEvent.putDouble("timestamp", actualNetEvent.timestamp);
        netEvent.putBoolean("isSupplement", actualNetEvent.isSupplement);
        netEvent.putString("type", actualNetEvent.type);
        netEvent.putString("method", actualNetEvent.method);
        netEvent.putString("error", actualNetEvent.error);
        netEvent.putString("url", actualNetEvent.url);
        netEvent.putDouble("size", actualNetEvent.getSize());
        netEvent.putInt("status", actualNetEvent.status);
        netEvent.putBoolean("isFiltered", actualNetEvent.isFiltered);

        if (actualNetEvent.custom != null) {
            netEvent.putString("body", actualNetEvent.custom.body);
            netEvent.putString("no_body_reason", actualNetEvent.custom.no_body_reason);
            netEvent.putMap("headers", Arguments.makeNativeMap(actualNetEvent.custom.headers));
        } else {
            netEvent.putString("body", null);
            netEvent.putString("no_body_reason", null);
            netEvent.putMap("headers", Arguments.createMap());
        }

        notifyClient(BGS_NETFLT_EVENT_NAME, netEvent);
    }

    private void processLogEvent(BugseeLog bugseeLog) {
        WritableMap logEvent = Arguments.createMap();

        logEvent.putString("text", bugseeLog.getMessage());
        logEvent.putInt("level", bugseeLog.getLevel().getIntValue());

        notifyClient(BGS_LOGLFT_EVENT_NAME, logEvent);
    }

    private void notifyClient(String eventName, WritableMap params) {
        this.getReactApplicationContext()
                .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                .emit(eventName, params);
    }

    private void notifyClient(String eventName, int value) {
        this.getReactApplicationContext()
                .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                .emit(eventName, value);
    }

    public static OkHttpClientFactory getDefaultHttpFactory() {
        return new DefaultNetworkInterceptorFactory();
    }

    @Override
    public String getName() {
        return "Bugsee";
    }

    @ReactMethod
    public void setFlag(String flagName, boolean flagState) {
        this.flags.put(flagName, flagState);
    }

    private void preprocessOptions(HashMap<String, Object> launchOptions) {
        if (launchOptions == null) {
            return;
        }

        if (launchOptions.containsKey("VideoQuality")) {
            try {
                Object rawVQ = launchOptions.get("VideoQuality");
                if (rawVQ != null) {
                    int index = (int) rawVQ;
                    VideoQuality vq = VideoQuality.values()[index];
                    launchOptions.put("VideoQuality", vq);
                }
            } catch (Throwable ignore) {}
        }

        if (launchOptions.containsKey("VideoMode")) {
            try {
                Object rawVM = launchOptions.get("VideoMode");
                if (rawVM != null) {
                    int index = (int) rawVM;
                    VideoMode vq = VideoMode.values()[index];
                    launchOptions.put("VideoMode", vq);
                }
            } catch (Throwable ignore) {}
        }

        if (launchOptions.containsKey("FallbackVideoMode")) {
            try {
                Object rawVM = launchOptions.get("FallbackVideoMode");
                if (rawVM != null) {
                    int index = (int) rawVM;
                    VideoMode vq = VideoMode.values()[index];
                    launchOptions.put("FallbackVideoMode", vq);
                }
            } catch (Throwable ignore) {}
        }
    }

    @ReactMethod
    public void launch(String appToken, ReadableMap options, Promise promise) {
        Activity activity = this.getCurrentActivity();

        if (activity != null) {
            if (options == null) {
                Bugsee.launch(activity, appToken);
                promise.resolve(true);
                return;
            }

            HashMap<String, Object> actualOptions = toMap(options, true);
            preprocessOptions(actualOptions);
            Bugsee.launch(activity, appToken, actualOptions);
            promise.resolve(true);
            return;
        }

        promise.resolve(false);
    }

    @ReactMethod
    public void stop(Promise promise) {
        Bugsee.stop();
        promise.resolve("");
    }

    @ReactMethod
    public void relaunch(ReadableMap options) {
        if (options == null) {
            Bugsee.relaunch();
            return;
        }

        HashMap<String, Object> actualOptions = toMap(options, true);
        Bugsee.relaunch(actualOptions);
    }

    @ReactMethod
    public void log(ReadableMap payload) {
        String text = RNUtils.optString(payload, "text", "");
        int level = RNUtils.optInt(payload, "level", 0);

        if (TextUtils.isEmpty(text)) {
            return;
        }

        BugseeLogLevel bugseeLogLevel;
        switch(level) {
            case 5:
                bugseeLogLevel = BugseeLogLevel.Verbose;
                break;
            case 4:
                bugseeLogLevel = BugseeLogLevel.Debug;
                break;
            case 3:
                bugseeLogLevel = BugseeLogLevel.Info;
                break;
            case 2:
                bugseeLogLevel = BugseeLogLevel.Warning;
                break;
            case 1:
                bugseeLogLevel = BugseeLogLevel.Error;
                break;
            default:
                bugseeLogLevel = BugseeLogLevel.Info;
                break;
        }

        Bugsee.log(text, bugseeLogLevel);
    }

    @ReactMethod
    public void event(ReadableMap payload) {
        String name = RNUtils.optString(payload, "name", "");

        ReadableMap params = null;
        if (payload.hasKey("params")) {
            params = payload.getMap("params");
        }

        if (TextUtils.isEmpty(name)) {
            return;
        }

        if (params != null) {
            Bugsee.event(name, toMap(params, false));
        } else {
            Bugsee.event(name);
        }
    }

    @ReactMethod
    public void trace(ReadableMap payload) {
        String name = RNUtils.optString(payload, "name", "");
        Object value = null;

        if (payload.hasKey("value"))
            value = getElementFromMap(payload, "value", false);

        if (TextUtils.isEmpty(name) || (value == null)) {
            return;
        }

        Bugsee.trace(name, value);
    }

    @ReactMethod
    public void pause() {
        Bugsee.pause();
    }

    @ReactMethod
    public void resume() {
        Bugsee.resume();
    }

    @ReactMethod
    public void toggleProtected(int viewId, boolean isProtected) {
        Activity activity = this.getCurrentActivity();

        if (activity != null) {
            View view = activity.findViewById(viewId);

            if (view != null) {
                if (isProtected) {
                    Bugsee.addSecureView(view);
                } else {
                    Bugsee.removeSecureView(view);
                }
            }
        }
    }

    @ReactMethod
    public void showReportDialog(ReadableMap payload) {
        String summary = RNUtils.optString(payload, "summary", "");
        String description = RNUtils.optString(payload, "description", "");
        int level = RNUtils.optInt(payload, "severity", 3);

        IssueSeverity severity;

        switch (level) {
            case 1:
                severity = IssueSeverity.VeryLow;
                break;
            case 2:
                severity = IssueSeverity.Medium;
                break;
            case 3:
                severity = IssueSeverity.High;
                break;
            case 4:
                severity = IssueSeverity.Critical;
                break;
            case 5:
                severity = IssueSeverity.Blocker;
                break;
            default:
                severity = IssueSeverity.Medium;
        }

        Bugsee.showReportDialog(summary, description, severity);
    }

    @ReactMethod
    public void upload(ReadableMap payload) {
        String summary = RNUtils.optString(payload, "summary", "");

        if (TextUtils.isEmpty(summary)) {
            summary = "";
        }

        String description = RNUtils.optString(payload, "description", "");
        int level = RNUtils.optInt(payload, "severity", 3);
        
        ArrayList<Object> objLabels = toList(payload.getArray("labels"));
        ArrayList<String> labels = new ArrayList<>(objLabels.size());
        for (Object object : objLabels) {
            if (object != null) {
                labels.add(object.toString());
            }
        }

        boolean includeVideo = RNUtils.optBoolean(payload, "includeVideo", true);

        IssueSeverity severity;

        switch (level) {
            case 1:
                severity = IssueSeverity.VeryLow;
                break;
            case 2:
                severity = IssueSeverity.Medium;
                break;
            case 3:
                severity = IssueSeverity.High;
                break;
            case 4:
                severity = IssueSeverity.Critical;
                break;
            case 5:
                severity = IssueSeverity.Blocker;
                break;
            default:
                severity = IssueSeverity.Medium;
        }

        Bugsee.upload(summary, description, severity, labels, includeVideo);
    }

    @ReactMethod
    public void setEmail(ReadableMap payload) {
        String email = RNUtils.optString(payload, "email", null);
        Bugsee.setEmail(email);
    }

    @ReactMethod
    public void clearEmail() {
        Bugsee.setEmail(null);
    }

    @ReactMethod
    public void getEmail(Promise promise) {
        promise.resolve(Bugsee.getEmail());
    }

    @ReactMethod
    public void setAttribute(ReadableMap payload) {
        String name = RNUtils.optString(payload, "name", "");

        if (TextUtils.isEmpty(name)) {
            return;
        }

        Object value = null;

        if (payload.hasKey("value")) {
            value = getElementFromMap(payload, "value", false);
        }

        if (value == null) {
            return;
        }

        Bugsee.setAttribute(name, value);
    }

    @ReactMethod
    public void getAttribute(String name, Promise promise) {
        Object attributeValue = Bugsee.getAttribute(name);
        promise.resolve(attributeValue);
    }

    @ReactMethod
    public void clearAttribute(ReadableMap payload) {
        String name = RNUtils.optString(payload, "name", "");

        if (TextUtils.isEmpty(name)) {
            return;
        }

        Bugsee.clearAttribute(name);
    }

    @ReactMethod
    public void clearAllAttributes() {
        Bugsee.clearAllAttributes();
    }

    @ReactMethod
    public void showFeedbackUI() {
        Bugsee.showFeedbackActivity(this.getCurrentActivity());
    }

    @ReactMethod
    public void setDefaultFeedbackGreeting(String message) {
        if (TextUtils.isEmpty(message)) {
            return;
        }

        Bugsee.setDefaultFeedbackGreeting(message);
    }

    @ReactMethod
    public void logException(String exceptionJson, boolean isHandled, ReadableMap options, Promise promise) throws ReactNativeWebException {
        ReactNativeWebException ex = new ReactNativeWebException(exceptionJson);

        if (isHandled) {
            if (options != null) {
                ExceptionLoggingOptions exOptions = new ExceptionLoggingOptions();
                exOptions.setIncludeVideo(!options.hasKey("includeVideo") || options.getBoolean("includeVideo"));
                exOptions.setExceptionDomain(options.getString("domain"));
                exOptions.setLabels(toStringList(options.getArray("labels")));
                if (options.hasKey("mergingRules")) {
                    ReadableMap mergingRules = options.getMap("mergingRules");
                    if (mergingRules != null) {
                        if (mergingRules.hasKey("skipFrames")) {
                            exOptions.Rules.setSkipFrames(mergingRules.getInt("skipFrames"));
                        }
                    }
                }
                Bugsee.logException(ex, exOptions);
            } else {
                Bugsee.logException(ex);
            }
            
        } else {
            Bugsee.onUncaughtException(Thread.currentThread(), ex);
        }

        promise.resolve("");
    }

    @ReactMethod
    public void deleteCollectedDataOnDevice(Promise promise) {
        Bugsee.deleteCollectedDataOnDevice((success) -> {
            promise.resolve(success);
        });
    }

    @ReactMethod
    public void addSecureRect(int left, int top, int width, int height) {
        Bugsee.addSecureRectangle(new Rect(left, top, left + width, top + height));
    }

    @ReactMethod
    public void removeSecureRect(int left, int top, int width, int height) {
        Bugsee.removeSecureRectangle(new Rect(left, top, left + width, top + height));
    }

    @ReactMethod
    public void getAllSecureRects(Promise promise) {
        ArrayList<Rect> secureRects = Bugsee.getAllSecureRectangles();
        int[][] rects = new int[secureRects.size()][];

        int i = 0;
        for (Rect rect : secureRects) {
            rects[i++] = new int[] { rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top };
        }

        promise.resolve(Arguments.makeNativeArray(rects));
    }

    @ReactMethod
    public void removeAllSecureRects() {
        Bugsee.removeAllSecureRectangles();
    }

    @ReactMethod
    public void getVersions(Promise promise) {
        try
        {
            PackageManager packageManager = this.reactContext.getPackageManager();
            PackageInfo info = packageManager.getPackageInfo(this.packageName, 0);

            WritableMap versions = Arguments.createMap();
            versions.putString("version", info.versionName);
            versions.putString("build", Integer.toString(info.versionCode));

            promise.resolve(versions);
        }
        catch (PackageManager.NameNotFoundException e)
        {
            promise.reject(e);
        }
    }

    @ReactMethod
    public void getLaunchOptions(final Promise promise) {
        HashMap<String, Object> launchOptions = BugseeInternalAdapter.getLaunchOptions();
        String jsonResult = "";

        try {
            JSONObject jsonOptions = new JSONObject(launchOptions);
            jsonResult = jsonOptions.toString();
        } catch (Exception e) {
            jsonResult = "";
        } finally {
            promise.resolve(jsonResult);
        }
    }

    @ReactMethod
    public void hideKeyboard(Boolean isHidden) {
        // Just a mock
    }

    @ReactMethod
    public void logNetworkEvent(String networkJson) {
        JSONObject parsedEvent;

        try {
            parsedEvent = new JSONObject(networkJson);
        } catch (JSONException e) {
            return;
        }

        ExchangeNetworkEvent netEvent = new ExchangeNetworkEvent();
        netEvent.timestamp = System.currentTimeMillis();
        netEvent.id = parsedEvent.optString("id", "");
        netEvent.type = parsedEvent.optString("type", "");
        netEvent.method = parsedEvent.optString("method", "");
        netEvent.url = parsedEvent.optString("url", "");
        netEvent.size = parsedEvent.optLong("size", 0);
        netEvent.status = parsedEvent.optInt("status", 0);
        netEvent.isFiltered = parsedEvent.optBoolean("isFiltered", false);
        netEvent.isSupplement = parsedEvent.optBoolean("isSupplement", false);

        netEvent.body = parsedEvent.isNull("body") ? null :
                parsedEvent.optString("body", "");
        netEvent.no_body_reason = parsedEvent.isNull("no_body_reason") ? null :
                parsedEvent.optString("no_body_reason", "");
        netEvent.error = parsedEvent.isNull("error") ? null :
                parsedEvent.optString("error", "");

        HashMap<String, String> headers = new HashMap<>();
        JSONObject headersObj = parsedEvent.optJSONObject("headers");

        if (headersObj != null) {
            Iterator<String> keys = headersObj.keys();
            while (keys.hasNext()) {
                String hKey = keys.next();
                headers.put(hKey, headersObj.optString(hKey, ""));
            }
        }

        netEvent.headers = headers;

        BugseeInternalAdapter.addNetworkEvent(netEvent);
    }

    @ReactMethod
    public void setAttachments(String attachmentsJson) {
        JSONObject parsedData;

        try {
            parsedData = new JSONObject(attachmentsJson);
        } catch (JSONException e) {
            return;
        }

        ArrayList<CustomAttachment> attachments = new ArrayList<>();

        JSONArray userAttachments = parsedData.optJSONArray("attachments");
        if (userAttachments == null) {
            BugseeInternalAdapter.setAttachments(attachments);
            return;
        }

        int count = userAttachments.length();

        for (int i = 0; i < count; ++i) {
            JSONObject item = userAttachments.optJSONObject(i);
            if (item != null) {
                CustomAttachment attachment = null;
                String fullPath = item.optString("fullpath", "");

                try {
                    if (!fullPath.equals("")) {
                        attachment = CustomAttachment.fromDataFilePath(fullPath);
                    } else {
                        String data = item.optString("data", "");
                        if (!data.equals("")) {
                            attachment = CustomAttachment.fromDataString(data);
                        }
                    }
                }
                catch (Exception e) {
                    String errorMessage = e.getMessage() != null ? e.getMessage() : "<no_message>";
                    Bugsee.log("Failed to handle attachment. Error follows" + errorMessage, BugseeLogLevel.Error);
                    attachment = null;
                }

                if (attachment != null) {
                    String fileName = item.optString("filename", "");
                    String name = item.optString("name", "");
                    attachment.setFileName(fileName);
                    attachment.setName(name);
                    attachments.add(attachment);
                }
            }
        }

        BugseeInternalAdapter.setAttachments(attachments);
    }

    @ReactMethod
    public void getAppearanceProperty(String propertyName, Promise promise) {
        BugseeAppearance bugseeAppearance = Bugsee.getAppearance();

        WritableMap appearance = Arguments.createMap();

        appearance.putInt("r", 0);
        appearance.putInt("g", 0);
        appearance.putInt("b", 0);
        appearance.putInt("a", 0);

        promise.resolve(appearance);
    }

    @ReactMethod
    public void setAppearanceProperty(ReadableMap appearanceData, Promise promise) {
        String propertyName = appearanceData.getString("name");
        int r = appearanceData.getInt("r");
        int g = appearanceData.getInt("g");
        int b = appearanceData.getInt("b");
        int a = appearanceData.getInt("a");
        Integer colorValue = Integer.valueOf((r << 24) | (g << 16) | (b << 8) | a);

        setClassPropertyValue(Bugsee.getAppearance(), propertyName, colorValue);

        promise.resolve("");
    }

    @ReactMethod
    public void testCrash() {
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                BugseeInternalAdapter.throwException();
            }
        }, 100 );
    }

    @ReactMethod
    public void addListener(String eventName) {
        // Keep: Required for RN built in Event Emitter Calls.
    }

    @ReactMethod
    public void removeListeners(Integer count) {
        // Keep: Required for RN built in Event Emitter Calls.
    }

    @ReactMethod
    public void captureViewHierarchy() {
        Bugsee.captureViewHierarchy();
    }
}
