//
//  RNBugsee.m
//  PropertyFinder
//
//  Created by Dmitry Fink on 9/8/16.
//  Copyright © 2016 Bugsee. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <React/RCTUIManager.h>

#import "RNBugsee.h"
#import "Bugsee/Bugsee.h"

typedef struct _BGSRect {
    float x;
    float y;
    float width;
    float height;
} BGSRect;

NSString* const BGS_NETFLT_EVENT_NAME = @"bgsNetFilterEvent";
NSString* const BGS_LOGLFT_EVENT_NAME = @"bgsLogFilterEvent";
NSString* const BGS_ATTACHMENTS_EVENT_NAME = @"bgsAttachmentsEvent";
NSString* const BGS_NEW_FEEDBACK_MESSAGES_EVENT_NAME = @"bgsNewFeedbackMessagesEvent";
NSString* const BGS_LIFECYCLE_EVENT_NAME = @"bgsLifecycleEvent";

@interface RNBugsee()
@property (nonatomic, strong) NSMutableDictionary * flags;
@property (nonatomic, strong) NSMutableDictionary * decisionBlocks;
@end

@implementation RNBugsee

RCT_EXPORT_MODULE(Bugsee);

// @synthesize bridge = _bridge;

+ (BOOL)requiresMainQueueSetup {
  return true;
}

- (id)init
{
  self = [super init];
  
  if (self) {
    _flags = [[NSMutableDictionary alloc] init];
    _decisionBlocks = [[NSMutableDictionary alloc] init];
    [Bugsee sharedInstance].delegate = self;
  }
  
  return self;
}

- (NSArray<NSString *> *)supportedEvents {
  return @[
      BGS_NETFLT_EVENT_NAME,
      BGS_LOGLFT_EVENT_NAME,
      BGS_ATTACHMENTS_EVENT_NAME,
      BGS_NEW_FEEDBACK_MESSAGES_EVENT_NAME,
      BGS_LIFECYCLE_EVENT_NAME
    ];
}

- (NSString *)serializeDictToJson:(NSDictionary *)dict
{
  NSError* jsonError;
  NSData* jsonData = [NSJSONSerialization dataWithJSONObject:dict
                                                     options:0
                                                       error:&jsonError];
  if (!jsonError && jsonData) {
    return [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
  }
  
  return nil;
}

- (void)bugseeFilterLog:(BugseeLogEvent *)log completionHandler:(BugseeLogFilterDecisionBlock)decisionBlock
{
  BOOL hasFilterCallback = [self getFlagValue:@"logFilter"];
  
  if (hasFilterCallback) {
    NSMutableDictionary* logJson = [[NSMutableDictionary alloc] init];
    logJson[@"timestamp"] = @(log.timestamp);
    logJson[@"text"] = log.text;
    logJson[@"level"] = [NSNumber numberWithInteger:log.level];
    
    NSString* jsonString = [self serializeDictToJson:logJson];
    
    if (jsonString) {
      [self sendEventWithName:BGS_LOGLFT_EVENT_NAME body:jsonString];
      decisionBlock(nil);
      return;
    }
  }

  decisionBlock(log);
}

-(void)bugsee:(Bugsee *)bugsee didReceiveNewFeedback:(NSArray<NSString *> *)messages
{
  BOOL hasFilterCallback = [self getFlagValue:@"feedbackHandler"];
  
  if (hasFilterCallback) {
    NSMutableDictionary* feedbackJson = [[NSMutableDictionary alloc] init];
    feedbackJson[@"messages"] = messages;

    NSString* jsonString = [self serializeDictToJson:feedbackJson];
    
    if (jsonString) {
      [self sendEventWithName:BGS_NEW_FEEDBACK_MESSAGES_EVENT_NAME body:jsonString];
    }
  }
}

-(void)bugseeFilterNetworkEvent:(BugseeNetworkEvent *)event completionHandler:(BugseeNetworkFilterDecisionBlock)decisionBlock{
  BOOL hasFilterCallback = [self getFlagValue:@"netFilter"];
  
  decisionBlock(hasFilterCallback ? nil : event);
  
  if (hasFilterCallback) {
    NSMutableDictionary* netJson = [[NSMutableDictionary alloc] init];
    netJson[@"timestamp"] = @(event.timestamp);
    netJson[@"type"] = event.bugseeNetworkEventType;
    netJson[@"id"] = event.ID;
    netJson[@"method"] = event.method;
    netJson[@"url"] = event.url;
    netJson[@"no_body_reason"] = event.noBodyReason;
    netJson[@"redirected_from"] = event.redirectedFromURL;
    netJson[@"status"] = [NSNumber numberWithInteger:event.responseCode];
    netJson[@"net_type"] = @(event.type);
    netJson[@"size"] = @(event.dataSize);
    netJson[@"headers"] = event.headers;
    netJson[@"body"] = [[NSString alloc] initWithData:event.body encoding:NSUTF8StringEncoding];
    netJson[@"error"] = event.error;
    netJson[@"override"] = @(event.override);
    
    NSString* jsonString = [self serializeDictToJson:netJson];
    
    if (jsonString) {
      [self sendEventWithName:BGS_NETFLT_EVENT_NAME body:jsonString];
    }
  }
}

-(void) bugseeAttachmentsForReport:(nonnull BugseeReport *)report completionHandler:(nonnull BugseeAttachmentsDecisionBlock)decisionBlock;
{
  // check whether handler was set in JS. We don't
  // want to waste time and cycles passing attachments
  // back and forth pointlessly
  BOOL hasAttachmentsCallback = [self getFlagValue:@"attachmentsHandler"];
  if (!hasAttachmentsCallback) {
    decisionBlock(nil);
    return;
  }

  // generate UUID to uniquely identify this event later
  NSString * eventId = [[NSUUID UUID] UUIDString];
  
  // generate report information block
  NSMutableDictionary* reportJson = [[NSMutableDictionary alloc] init];
  reportJson[@"severity"] = @(report.severity);
  reportJson[@"type"] = report.type;
  
  // generate event object with report data
  NSMutableDictionary* attachmentsJson = [[NSMutableDictionary alloc] init];
  attachmentsJson[@"report"] = reportJson;
  attachmentsJson[@"id"] = eventId;
  
  // serialize to string (as JSON)
  NSString* jsonString = [self serializeDictToJson:attachmentsJson];
  
  [_decisionBlocks setObject:decisionBlock forKey:eventId];
  
  // finally, send event to JS land
  [self sendEventWithName:BGS_ATTACHMENTS_EVENT_NAME body:jsonString];
}

-(void) bugseeLifecycleEvent:(BugseeLifecycleEventType)eventType {
  BOOL hasLifecycleCallback = [self getFlagValue:@"lifecycleHandler"];
  if (!hasLifecycleCallback) {
    return;
  }

  [self sendEventWithName:BGS_LIFECYCLE_EVENT_NAME body:@(eventType)];
}

- (BOOL)getFlagValue:(NSString *)forName {
  id rawFlagValue = [_flags objectForKey:forName];
  
  if (rawFlagValue) {
    return [rawFlagValue boolValue];
  }
  
  return false;
}

RCT_EXPORT_METHOD(setAttachments:(nonnull NSString *)attachmentsJson)
{
  // convert NSString to NSData to later use in JSON
  // decoding
  NSData *jsonData = [attachmentsJson dataUsingEncoding:NSUTF8StringEncoding];
  // variable to store JSON decoding error
  NSError *error = nil;
  
  // parse JSON
  NSDictionary *parsedJson = [NSJSONSerialization JSONObjectWithData:jsonData options:0 error:&error];
  
  if (error) {
    // Invalid JSON?
    return;
  }
  
  // get event ID. We need it find target decisionBlock
  // to pass attachments to
  NSString * eventId = parsedJson[@"id"];
  NSArray<NSDictionary*>* rawAttachments = [parsedJson objectForKey:@"attachments"];

  BugseeAttachmentsDecisionBlock decisionBlock = (BugseeAttachmentsDecisionBlock)[_decisionBlocks objectForKey:eventId];
  if (decisionBlock) {
    // remove this stored block handle instantly
    [_decisionBlocks removeObjectForKey:eventId];
    
    // initialize new array of our attachments
    NSMutableArray<BugseeAttachment*>* attachments = [[NSMutableArray new] init];
    
    // iterate through all the raw attachments and convert
    // them into Bugsee attachments
    for (NSDictionary* rawAttachment in rawAttachments) {
      NSString * attachmentName = [rawAttachment objectForKey:@"name"];
      NSString * attachmentFileName = [rawAttachment objectForKey:@"filename"];
      NSString * attachmentPath = [rawAttachment objectForKey:@"fullpath"];
      NSString * attachmentData = [rawAttachment objectForKey:@"data"];
        
      BugseeAttachment * attachment = nil;

      // take only those attachments having all the fields
      if (attachmentName && attachmentFileName) {
        if (attachmentPath) {
            attachment = [BugseeAttachment attachmentWithName:attachmentName filename:attachmentFileName data:[NSData dataWithContentsOfFile:attachmentPath]];
        } else if (attachmentData) {
            attachment = [BugseeAttachment attachmentWithName:attachmentName filename:attachmentFileName data:[attachmentData dataUsingEncoding:NSUTF8StringEncoding]];
        }
      }

      // make sure attachment was properly created and
      // is not nil
      if (attachment) {
        [attachments addObject:attachment];
      }
    }

    // finally, call the block with constructed attachments
    decisionBlock(attachments);
  }
}

RCT_EXPORT_METHOD(setFlag:(NSString *)flag flagState:(BOOL)flagState)
{
  id value = flagState ? @YES : @NO;
  
  [_flags setObject:value forKey:flag];
}

RCT_REMAP_METHOD(launch,
                 token:(NSString *)token
                 options:(NSDictionary *)options
                 launch_resolver:(RCTPromiseResolveBlock)resolve
                 launch_rejecter:(RCTPromiseRejectBlock)reject)
{
  BugseeOptions * finalOptions = [BugseeOptions optionsFrom:options];

  if ([NSThread isMainThread]) {
    [Bugsee launchWithToken:token options:finalOptions started:^(BOOL success) {
      resolve(@(success));
    }];
  } else {
    dispatch_sync(dispatch_get_main_queue(), ^{
      [Bugsee launchWithToken:token options:finalOptions started:^(BOOL success) {
        resolve(@(success));
      }];
    });
  }
}

RCT_REMAP_METHOD(relaunch,
                 options:(NSDictionary *)options
                 relaunch_resolver:(RCTPromiseResolveBlock)resolve
                 relaunch_rejecter:(RCTPromiseRejectBlock)reject)
{
  BugseeOptions * finalOptions = [BugseeOptions optionsFrom:options];

  [Bugsee relaunchWithOptions:finalOptions started:^(BOOL success) {
    resolve(@(success));
  }];
}

RCT_REMAP_METHOD(stop,
                 stop_resolver:(RCTPromiseResolveBlock)resolve
                 stop_rejecter:(RCTPromiseRejectBlock)reject) {
  [Bugsee stop:^(void) {
    resolve(@(YES));
  }];
}

RCT_EXPORT_METHOD(log:(NSDictionary *)params)
{
  NSString *text = params[@"text"];
  
  int logLevel = params[@"level"] ?
  [params[@"level"] intValue] :
  BugseeLogLevelInfo;
  int64_t timestamp = params[@"timestamp"] ?
  [params[@"timestamp"] longLongValue] :
  ((CFAbsoluteTimeGetCurrent() + NSTimeIntervalSince1970) * 1000);
  
  [Bugsee log:text level:logLevel timestamp:timestamp];
}

RCT_EXPORT_METHOD(event:(NSDictionary *)params_)
{
  NSString *name = params_[@"name"];
  NSDictionary *params = params_[@"params"];
  if (name) {
    if (params) {
      [Bugsee registerEvent:name withParams:params];
    } else {
      [Bugsee registerEvent:name];
    }
  } else {
    RCTLogError(@"Missing name for the event!");
  }
}

RCT_EXPORT_METHOD(trace:(NSDictionary *)params_)
{
  NSString *name = params_[@"name"];
  NSObject *value = params_[@"value"];
  if (name && value) {
    [Bugsee traceKey:name withValue:value];
  } else {
    RCTLogError(@"Missing name or value for the trace!");
  }
  
}

RCT_EXPORT_METHOD(showReportDialog:(NSDictionary *)params_)
{
  NSString *summary = params_[@"summary"];
  NSString *description = params_[@"description"];
  int severity = params_[@"severity"] ? [params_[@"severity"] intValue] : 3;
  
  if (summary) {
    [Bugsee showReportControllerWithSummary:summary description:description severity:severity];
  } else {
    [Bugsee showReportController];
  }
}

RCT_EXPORT_METHOD(upload:(NSDictionary *)params_)
{
  NSString *summary = params_[@"summary"];
  NSString *description = params_[@"description"];
  int severity = params_[@"severity"] ? [params_[@"severity"] intValue] : 3;
  NSArray * labels = params_[@"labels"] ? params_[@"labels"] : nil;
  BOOL includeVideo = params_[@"includeVideo"] ? [params_[@"includeVideo"] boolValue] : YES;

  summary = summary ?: @"";

  [Bugsee uploadWithSummary:summary
                description:description
                   severity:severity
                     labels:labels
               includeVideo:includeVideo];
}

RCT_EXPORT_METHOD(setEmail:(NSDictionary *)params_)
{
  NSString *email = params_[@"email"];
  [Bugsee setEmail:email];
}

RCT_REMAP_METHOD(getEmail,
                 getEmail_resolver:(RCTPromiseResolveBlock)resolve
                 getEmail_rejecter:(RCTPromiseRejectBlock)reject)
{
  NSString *email = [Bugsee getEmail];
  resolve(email);
}

RCT_EXPORT_METHOD(clearEmail)
{
  [Bugsee clearEmail];
}

RCT_EXPORT_METHOD(setAttribute:(NSDictionary *)params_)
{
  NSString *name = params_[@"name"];
  NSObject *value = params_[@"value"];
  if (name && value) {
    [Bugsee setAttribute:name withValue:value];
  } else {
    RCTLogError(@"Missing name or value for the attribute!");
  }
}

RCT_EXPORT_METHOD(clearAttribute:(NSDictionary *)params_)
{
  NSString *name = params_[@"name"];
  if (name) {
    [Bugsee clearAttribute:name];
  } else {
    RCTLogError(@"Missing name for the attribute!");
  }
}

RCT_EXPORT_METHOD(clearAllAttributes)
{
  [Bugsee clearAllAttributes];
}

RCT_REMAP_METHOD(getAttribute,
                 name:(NSString *)name
                 getAttribute_resolver:(RCTPromiseResolveBlock)resolve
                 getAttribute_rejecter:(RCTPromiseRejectBlock)reject)
{
  id attributeValue = [Bugsee getAttribute:name];
  resolve(attributeValue);
}

RCT_EXPORT_METHOD(showFeedbackUI)
{
  [Bugsee showFeedbackController];
}

RCT_EXPORT_METHOD(setDefaultFeedbackGreeting:(NSString *)message)
{
  if (message) {
    [Bugsee setDefaultFeedbackGreeting:message];
  } else {
    RCTLogError(@"Missing message for the greeting!");
  }
}

RCT_EXPORT_METHOD(pause)
{
  [Bugsee pause];
}

RCT_EXPORT_METHOD(resume)
{
  [Bugsee resume];
}

RCT_EXPORT_METHOD(toggleProtected:(nonnull NSNumber *)reactTag isProtected:(BOOL)isProtected)
{
  if (self.bridge != nil)
  {
    dispatch_async(self.bridge.uiManager.methodQueue, ^{
      [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
        __block UIView *view = viewRegistry[reactTag];
        if (view) {
          [Bugsee setView:view asHidden:isProtected];
        }
      }];
    });
  }
}

RCT_REMAP_METHOD(logException,
                 logException:(nonnull NSString *)exceptionJson
                 isHandled:(BOOL)isHandled
                 options:(NSDictionary * _Nullable)options
                 logException_resolver:(RCTPromiseResolveBlock)resolve
                 logException_rejecter:(RCTPromiseRejectBlock)reject)
{
  if (isHandled) {
      BugseeExceptionLoggingOptions * exOptions = nil;
      if (options) {
          exOptions = [[BugseeExceptionLoggingOptions alloc] init];
          exOptions.includeVideo = options[@"includeVideo"] ? [options[@"includeVideo"] boolValue] : YES;
          exOptions.labels = options[@"labels"];
          exOptions.mergingRules = options[@"mergingRules"];
      }
      [Bugsee logException:@"ReactNativeWebException"
                    reason:exceptionJson
                   options:exOptions
                completion:^{
                    resolve(@"");
                }];
  } else {
      [Bugsee logUnhandledException:@"ReactNativeWebException"
                             reason:exceptionJson
                         completion:^{
                            resolve(@"");
                         }];
  }
}

RCT_REMAP_METHOD(getVersions,
                 getVersions_resolver:(RCTPromiseResolveBlock)resolve
                 getVersions_rejecter:(RCTPromiseRejectBlock)reject)
{
  NSBundle *bundle = [NSBundle mainBundle];
  NSString *version = [bundle objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
  NSString *build = [bundle objectForInfoDictionaryKey:@"CFBundleVersion"];
  
  NSMutableDictionary *versions = [[NSMutableDictionary alloc] init];
  [versions setObject:version forKey:@"version"];
  [versions setObject:build forKey:@"build"];
  
  resolve(versions);
}

RCT_EXPORT_METHOD(addSecureRect:(NSInteger)x
                  y:(NSInteger)y
                  width:(NSInteger)width
                  height:(NSInteger)height)
{
  [Bugsee addSecureRect:CGRectMake(x, y, width, height)];
}

RCT_EXPORT_METHOD(removeSecureRect:(NSInteger)x
                  y:(NSInteger)y
                  width:(NSInteger)width
                  height:(NSInteger)height)
{
  [Bugsee removeSecureRect:CGRectMake(x, y, width, height)];
}

RCT_REMAP_METHOD(getAllSecureRects,
                 getAllRects_resolver:(RCTPromiseResolveBlock)resolve
                 getAllRects_rejecter:(RCTPromiseRejectBlock)reject)
{
  NSArray * secureRects = [Bugsee getAllSecureRects];
  NSMutableArray *rects = [[NSMutableArray alloc] init];
  
  for (NSValue * rectValue in secureRects){
    BGSRect rect;
    [rectValue getValue:&rect];
    
    NSMutableArray* points = [[NSMutableArray alloc] init];
    [points addObject:[NSNumber numberWithInteger:rect.x]];
    [points addObject:[NSNumber numberWithInteger:rect.y]];
    [points addObject:[NSNumber numberWithInteger:rect.width]];
    [points addObject:[NSNumber numberWithInteger:rect.height]];
    
    [rects addObject:points];
  }
  
  resolve(rects);
}

RCT_EXPORT_METHOD(removeAllSecureRects)
{
  [Bugsee removeAllSecureRects];
}

RCT_EXPORT_METHOD(hideKeyboard:(BOOL)isHidden)
{
  [Bugsee hideKeyboard:isHidden];
}

RCT_REMAP_METHOD(getLaunchOptions,
                 getLaunchOptions_resolver:(RCTPromiseResolveBlock)resolve
                 getLaunchOptions_rejecter:(RCTPromiseRejectBlock)reject)
{
  NSDictionary *options = [Bugsee getLaunchOptions];
  
  resolve(options);
}

RCT_EXPORT_METHOD(logNetworkEvent:(nonnull NSString *)eventJson) {
  NSData *jsonData = [eventJson dataUsingEncoding:NSUTF8StringEncoding];
  NSError *error = nil;
  
  NSDictionary *parsedJson = [NSJSONSerialization JSONObjectWithData:jsonData options:0 error:&error];
  
  if (error) {
    // Invalid JSON?
    return;
  }
  
  NSTimeInterval timestamp;
  BugseeNetworkType net_type;
  BOOL isFiltered = [parsedJson[@"isFiltered"] boolValue];
  
  timestamp = [parsedJson[@"timestamp"] doubleValue];
  
  if (!timestamp) {
    timestamp = ((CFAbsoluteTimeGetCurrent() + NSTimeIntervalSince1970) * 1000);
  }
  
  net_type = [parsedJson[@"net_type"] integerValue];
  
  if (!net_type) {
    net_type = BugseeNetwork;
  }
  
  BugseeNetworkEvent *bugseeNetEvent = [BugseeNetworkEvent new];
  bugseeNetEvent.timestamp = timestamp;
  bugseeNetEvent.type = net_type;
  bugseeNetEvent.ID = parsedJson[@"id"];
  bugseeNetEvent.bugseeNetworkEventType = parsedJson[@"type"];
  bugseeNetEvent.method = parsedJson[@"method"];
  bugseeNetEvent.url = parsedJson[@"url"];
  bugseeNetEvent.dataSize = [parsedJson[@"size"] longLongValue];
  bugseeNetEvent.headers = parsedJson[@"headers"];
  bugseeNetEvent.noBodyReason = parsedJson[@"no_body_reason"];
  bugseeNetEvent.redirectedFromURL = parsedJson[@"redirected_from"];
  bugseeNetEvent.responseCode = [parsedJson[@"status"] integerValue];
  bugseeNetEvent.override = [parsedJson[@"override"] boolValue];
  
  NSString *bodyStr = parsedJson[@"body"];
  if (bodyStr) bugseeNetEvent.body = [bodyStr dataUsingEncoding:NSUTF8StringEncoding];
  
  id errObj = parsedJson[@"error"];
  
  if (errObj) {
    if ([errObj isKindOfClass:[NSDictionary class]]) {
      bugseeNetEvent.error = errObj;
    } else if ([errObj isKindOfClass:[NSString class]]) {
      id keys[] = { @"error" };
      id values[] = { errObj };
      bugseeNetEvent.error = [NSDictionary dictionaryWithObjects:values forKeys:keys count:1];
    }
  }
  
  [Bugsee registerNetworkEvent:bugseeNetEvent needsToBeFiltered:!isFiltered];
}

RCT_EXPORT_METHOD(testCrash)
{
  dispatch_async(dispatch_get_main_queue(), ^{
    [Bugsee testExceptionCrash];
  });
}

RCT_EXPORT_METHOD(setAppearanceProperty:(nonnull NSDictionary *)appearanceData) {
  NSString * propertyName = appearanceData[@"name"];
  NSInteger r = [appearanceData[@"r"] intValue];
  NSInteger g = [appearanceData[@"g"] intValue];
  NSInteger b = [appearanceData[@"b"] intValue];
  NSInteger a = [appearanceData[@"a"] intValue];

  UIColor * color = [UIColor colorWithRed:r/255.0 green:g/255.0 blue:b/255.0 alpha:a/255.0];

  [[Bugsee appearance] setValue:color forKey:propertyName];
}

RCT_REMAP_METHOD(getAppearanceProperty,
                 propertyName:(nonnull NSString *)propertyName
                 getAppearanceProperty_resolver:(RCTPromiseResolveBlock)resolve
                 getAppearanceProperty_rejecter:(RCTPromiseRejectBlock)reject) {
    @try {
        const UIColor * propertyColor = (UIColor *)[[Bugsee appearance] valueForKey:propertyName];
        if (propertyColor) {
            CGFloat red = 0.0, green = 0.0, blue = 0.0, alpha = 0.0;
            [propertyColor getRed:&red green:&green blue:&blue alpha:&alpha];
            NSDictionary * colorMap = @{
                @"r": @(red),
                @"g": @(green),
                @"b": @(blue),
                @"a": @(alpha)
            };
            resolve(colorMap);
            return;
        }
    } @catch (NSException * exception) {
        NSLog(@"Bugsee: Failed to get color property value with exception: %s", exception.reason.UTF8String);
    }
    
    resolve(nil);
}

RCT_EXPORT_METHOD(captureViewHierarchy)
{
  [Bugsee captureViewHierarchy];
}

RCT_REMAP_METHOD(deleteCollectedDataOnDevice,
                 deleteCollectedDataOnDevice_resolver:(RCTPromiseResolveBlock)resolve
                 deleteCollectedDataOnDevice_rejecter:(RCTPromiseRejectBlock)reject) {
                  [Bugsee deleteCollectedDataOnDevice:^(BOOL success) {
                    resolve(@(success));
                  }];
                 }
@end

