/* eslint-disable */
'use strict';

const { spawnSync, execSync } = require('child_process');
const { dirname, resolve, join } = require('path');
const { existsSync, readFileSync } = require('fs');

function logEvent(...args) {
    args.unshift('[BUGSEE]');
    console.log(...args);
}

function warnEvent(...args) {
    args.unshift('[BUGSEE]');
    console.warn(...args);
}

function findBugseeSourcemapsToolPath() {
    const originalBugseeDir = process.env.BUGSEE_ORIGINAL_DIR;
   
    // try resolving the pacakge first
    try {
        const resolvedPath = require.resolve('bugsee-sourcemaps');
        if (resolvedPath) {
            if (resolvedPath.endsWith('bugsee-sourcemaps')) {
                return join(resolvedPath, 'cli.js');
            } else {
                return join(dirname(resolvedPath), 'cli.js');
            }
        }
    } catch (e) {}

    // then, try to list it via "npm ls"
    try {
        const options = {};
        if (originalBugseeDir) {
            options.cwd = dirname(originalBugseeDir);
        }
        const p = spawnSync('npm', ['ls', '--parseable', 'bugsee-sourcemaps'], options);
        if (p && p.stdout) {
            const firstResult = (p.stdout.toString('utf-8') || '').split('\n')[0];
            if (firstResult) {
                return join(firstResult, 'cli.js')
            }
        }
    } catch (e) {}

    // then, try to find globally installed tool
    try {
        const whichResult = execSync('which bugsee-sourcemaps');
        if (whichResult) {
            return (whichResult || '').toString().replace('\n', '');
        }
    } catch (e) {}

    // finally, fallback to static path
    if (originalBugseeDir) {
        return join(dirname(originalBugseeDir), 'node_modules', 'react-native-bugsee', 'node_modules', 'bugsee-sourcemaps', 'cli.js');
    }

    return null;
}

function parseProperties(lines) {
    const properties = {};

    lines.forEach(function (line) {                    
        // strip any whitespace or tabs at the start of the line
        const cleanLine = line.replace(/^[\s\t]+/, '');
        // make sure this line is not commented out
        if (cleanLine && (cleanLine[0] !== '#')) {
            const equalIndex = cleanLine.indexOf('=');
            // find first equality sign position
            if (equalIndex !== -1) {
                const key = cleanLine.substring(0, equalIndex).trim();
                const value = cleanLine.substring(equalIndex + 1);

                properties[key] = value;
            }
        }
    });

    return properties;
}

function loadProperties() {
    let propertiesFilePath = '';

    if (process.env.BUGSEE_PROPERTIES) {
        propertiesFilePath = resolve(join(process.env.BUGSEE_ORIGINAL_DIR, process.env.BUGSEE_PROPERTIES));
    }
    if (!propertiesFilePath || !existsSync(propertiesFilePath)) {
        warnEvent(`Properties file was not found at "${propertiesFilePath}"! Falling back to static path lookup.`);
        propertiesFilePath = join(process.env.PROJECT_DIR, 'bugsee.properties');
    }

    if (!propertiesFilePath || !existsSync(propertiesFilePath)) {
        warnEvent(`Properties file was not found at "${propertiesFilePath}"!`);
        return null;
    }

    try {
        const propertiesFileContent = readFileSync(propertiesFilePath).toString('utf-8');
        if (propertiesFileContent) {
            return parseProperties(propertiesFileContent.split('\n'));
        }
    } catch (e) {
        warnEvent(`Failed to load properties file "${propertiesFilePath}"!`, e);
    }

    return null;
}

function loadPlist() {
    const projectDir = process.env.PROJECT_DIR;
    const plistRelPath = process.env.INFOPLIST_FILE;
    let absPlistPath = join(projectDir, plistRelPath);
    if (!absPlistPath || !existsSync(absPlistPath)) {
        warnEvent(`Info.plist was not found at "${absPlistPath}" provided by 'PROJECT_DIR + INFOPLIST_FILE'! Trying another location.`);
        absPlistPath = process.env.PRODUCT_SETTINGS_PATH;
    }
    if (!absPlistPath || !existsSync(absPlistPath)) {
        warnEvent(`Info.plist was not found at "${absPlistPath}" provided by 'PRODUCT_SETTINGS_PATH' too! Failed to find the Info.plist.`);
        return null;
    }

    try {
        return readFileSync(absPlistPath).toString('utf-8');
    } catch (e) {
        warnEvent(`Failed to load and read Info.plist at "${absPlistPath}"!`, e);
    }

    return null;
}

function getBuildArguments() {
    const result = {};

    /*
        bundle
        --entry-file index.js
        --platform ios
        --dev true
        --reset-cache
        --bundle-output /path/to/main.jsbundle
        --assets-dest /path/to/app.app
        --sourcemap-output /path/to/main.jsbundle.map
    */

    let sourceMapOverriden = false;
    for (let index = 0; index < process.argv.length; index++) {
        const element = process.argv[index];

        switch (element) {
            case '--sourcemap-output':
                if (!sourceMapOverriden) {
                    result.sourcemapPath = process.argv[index + 1];
                }
                break;
            
            case '--source-map-override':
                result.sourcemapPath = process.argv[index + 1];
                sourceMapOverriden = true;
                break;

            case '--bundle-output':
                result.bundlePath = process.argv[index + 1];
                break;

            case '--dev':
                result.dev = process.argv[index + 1] === 'true';
                break;

            case '--platform':
                result.platform = process.argv[index + 1];
                break;
        }
    }

    if (result.dev === undefined || result.dev === null) {
        result.dev = (process.env.CONFIGURATION || '').toLowerCase() === 'debug';
    }

    return result;
}

function getAppToken(platform, properties) {
    // check environment variable first
    if (process.env.BUGSEE_APP_TOKEN) {
        return process.env.BUGSEE_APP_TOKEN;
    } else {
        logEvent('"BUGSEE_APP_TOKEN" environment variable is not set. Looking for application token in other sources!');
    }

    if (properties && properties['app.token']) {
        return properties['app.token'];
    } else {
        if (properties) {
            logEvent('"app.token" key was not found in properties file. Looking for application token in other sources!');
        }
    }

    return null;
}

function getValueFromPlist(plistContent, keyName) {
    const regex = new RegExp(`(?:<key>${keyName}</key>\\n\\s*\\t*<\\w+>)([^<]+)(?:</\\w+>)`);
    regex.lastIndex = 0;
    const matches = regex.exec(plistContent);
    const matchedValue = matches ? matches[1] : null;

    if (matchedValue && matchedValue[0] === '$') {
        const cleanMatchedEnvName = matchedValue.substring(2, matchedValue.length - 1);
        logEvent(`Found clean value "${cleanMatchedEnvName}" for key "${keyName}"!`);
        if (process.env[cleanMatchedEnvName]) {
            return process.env[cleanMatchedEnvName];
        }
    }

    return matchedValue;
}

function getAppVersion(platform, properties) {
    // check environment variable first
    if (process.env.BUGSEE_APP_VERSION) {
        return process.env.BUGSEE_APP_VERSION;
    } else {
        logEvent('"BUGSEE_APP_VERSION" environment variable is not set. Looking for application version in other sources!');
    }

    // then, check properties
    if (properties && properties['app.version']) {
        return properties['app.version'];
    } else {
        if (properties) {
            logEvent('"app.version" key was not found in properties file. Looking for application version in other sources!');
        }
    }

    if (platform === 'ios') {
        const plistContent = loadPlist();   
        if (plistContent) {
            return getValueFromPlist(plistContent, 'CFBundleShortVersionString');
        }
    }

    return null;
}

function getConfiguration(buildArguments) {
    // check build arguments first
    if (buildArguments.dev !== undefined && buildArguments.dev !== null) {
        return buildArguments.dev ? 'debug' : 'release';
    }

    // then check the environment variable
    if (process.env.CONFIGURATION) {
        return (/debug/i).test(process.env.CONFIGURATION) ? 'debug' : 'release';
    }

    // finally, fallback to debug
    return 'debug';
}

function main() {
    if (
        !process.argv ||
        (process.argv.indexOf('bundle') === -1 &&
        process.argv.indexOf('ram-bundle') === -1 &&
        process.argv.indexOf('--source-map-override') === -1)
        ) {
        // hook into correct task
        return;
    }

    const bugseeSourcemapsToolPath = findBugseeSourcemapsToolPath();
    if (bugseeSourcemapsToolPath) {
        const buildArguments = getBuildArguments();
        const platform = buildArguments.platform || 'ios';
        const properties = loadProperties();
        const nodeBinary = process.env.BUGSEE_ORIGINAL_NODE_BINARY || process.argv[0];
        const appVersion = getAppVersion(platform, properties);
        const appToken = getAppToken(platform, properties);
        const configuration = getConfiguration(buildArguments);
        const endpoint = process.env.BUGSEE_ENDPOINT || (properties && properties.endpoint);

        const bugseeSourcemapsToolArgs = [
            bugseeSourcemapsToolPath,
            'upload',
            '--overwrite',
            '--app-token', appToken,
            '--app-version', appVersion,
            '--configuration', configuration,
            '--platform', platform,
            '--source-map', buildArguments.sourcemapPath
        ];

        if (buildArguments.bundlePath) {
            bugseeSourcemapsToolArgs.push('--bundle', buildArguments.bundlePath);
        }

        if (endpoint) {
            bugseeSourcemapsToolArgs.push('--endpoint', endpoint);
        }

        //
        // Uncomment lines below to debug the gathered arguments
        //
        // logEvent(bugseeSourcemapsToolArgs);
        // process.exit(1);

        const p = spawnSync(nodeBinary, bugseeSourcemapsToolArgs);
        if (p.error) {
            console.error(p.error);
            process.exit(1);
        } else {
            logEvent(p.stdout.toString());
            logEvent(p.stderr.toString());
        }
    } else {
        warnEvent('"bugsee-sourcemaps" tool was not found. Please make sure it is properly installed');
    }
}

main();
