const hexRgx = /[A-Za-z0-9]/;
export function isDevMode() {
    return typeof __DEV__ !== 'undefined' && !!__DEV__;
}
/**
 * Checks whether specified value is Function
 * @param val {*}
 * @returns {boolean}
 */
// eslint-disable-next-line no-unused-vars
export function isFunction(val) {
    return typeof val === 'function';
}
export function isUndefined(val) {
    return val === undefined;
}
export function isNull(val) {
    return val === null;
}
export function isNil(val) {
    return val === undefined || val === null;
}
export function isArray(val) {
    return !isNil(val) && Array.isArray(val);
}
export function isObject(val) {
    return !isNil(val) && !isArray(val) && typeof val === 'object';
}
export function isPrimitive(val) {
    if (isNil(val)) {
        return true;
    }
    const type = typeof val;
    // in this implementation, we treat 'Symbol' as primitive
    return type !== 'function' && type !== 'object';
}
export function first(val) {
    if (isNil(val) || !Array.isArray(val)) {
        return undefined;
    }
    if (!val.length) {
        return undefined;
    }
    return val[0];
}
export function extend(...objs) {
    const dst = objs[0] || {};
    for (let i = 1; i < objs.length; i += 1) {
        const src = objs[i];
        if (src) {
            Object.assign(dst, src);
        }
    }
    return dst;
}
/**
 *
 * @param collection
 * @param val
 * @returns {boolean}
 */
export function includes(collection, val) {
    const len = collection.length;
    for (let i = 0; i < len; i += 1) {
        if (collection[i] === val) {
            return true;
        }
    }
    return false;
}
export function pick(obj, fields) {
    if (isNil(obj) || isNil(fields)) {
        return obj;
    }
    const newObj = {};
    for (let i = 0; i < fields.length; i += 1) {
        const field = fields[i];
        if (field in obj) {
            newObj[field] = obj[field];
        }
    }
    return newObj;
}
export function omit(obj, fields) {
    if (isNil(obj) || isNil(fields)) {
        return obj;
    }
    const newObj = {};
    Object.keys(obj).forEach((key) => {
        if (!includes(fields, key)) {
            newObj[key] = obj[key];
        }
    });
    return newObj;
}
export function clone(obj, isDeep) {
    if (isNil(obj)) {
        return obj;
    }
    if (!isObject(obj) && !isArray(obj)) {
        // we can't clone primitives
        return obj;
    }
    if (!isDeep && isObject(obj)) {
        return { ...obj };
    }
    if (!isDeep && isArray(obj)) {
        return [...obj];
    }
    if (isArray(obj)) {
        return obj.map((value) => isObject(value) || isArray(value) ? clone(value, isDeep) : value);
    }
    const output = {};
    const keys = Object.keys(obj);
    for (let index = 0; index < keys.length; index += 1) {
        const key = keys[index];
        const value = obj[key];
        output[key] = isObject(value) || isArray(value) ? clone(value, isDeep) : value;
    }
    return output;
}
/**
 *
 * @private
 * @returns {String}
 */
function s4() {
    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1); // eslint-disable-line no-bitwise
}
/**
 *
 * @private
 * @param replacer
 * @param cycleReplacer
 * @returns {function(*=, *=): *}
 */
function serializer(replacer, cycleReplacer) {
    const stack = [];
    const keys = [];
    const finalCycleReplacer = cycleReplacer ??
        function cycleReplacerFallback(key, value) {
            if (stack[0] === value) {
                return '[Circular ~]';
            }
            return `[Circular ~.${keys.slice(0, stack.indexOf(value)).join('.')}]`;
        };
    return function innerSerializer(key, value) {
        if (stack.length > 0) {
            const thisPos = stack.indexOf(this);
            if (thisPos > -1) {
                stack.splice(thisPos + 1);
                keys.splice(thisPos, Infinity, key);
            }
            else {
                stack.push(this);
                keys.push(key);
            }
            if (stack.indexOf(value) > -1) {
                // eslint-disable-next-line no-param-reassign
                value = finalCycleReplacer.call(this, key, value);
            }
        }
        else {
            stack.push(value);
        }
        return replacer ? replacer.call(this, key, value) : value;
    };
}
/**
 * Return GUID (Globally Unique IDentifier)
 * @returns {string}
 */
export function guid() {
    return `${s4()}${s4()}-${s4()}-${s4()}-${s4()}-${s4()}${s4()}${s4()}`;
}
/**
 * Converts specified JS object into JSON string representation
 * and properly handles circular references in nested properties
 *
 * @param obj {*} Object to stringify
 * @param [replacer] {Function}
 * @param [spaces] {String}
 * @param [cycleReplacer] {Function}
 * @returns {string}
 */
export function jsonStringify(obj, replacer, spaces, cycleReplacer) {
    return JSON.stringify(obj, serializer(replacer, cycleReplacer), spaces);
}
/**
 * Stringify any value to JSON
 *
 * @param value
 * @returns
 */
export function stringify(value) {
    const argType = typeof value;
    if (argType === 'string') {
        return value;
    }
    if (Array.isArray(value) || argType === 'object') {
        return jsonStringify(value);
    }
    return String(argType);
}
/* eslint-disable */
function _ad(r, t) {
    var a = (65535 & r) + (65535 & t);
    return (((r >> 16) + (t >> 16) + (a >> 16)) << 16) | (65535 & a);
}
function _rl(r, t) {
    return (r << t) | (r >>> (32 - t));
}
function _ft(r, t, a, n) {
    return r < 20
        ? (t & a) | (~t & n)
        : r < 40
            ? t ^ a ^ n
            : r < 60
                ? (t & a) | (t & n) | (a & n)
                : t ^ a ^ n;
}
function _kt(r) {
    return r < 20 ? 1518500249 : r < 40 ? 1859775393 : r < 60 ? -1894007588 : -899497514;
}
function _rth(r) {
    for (var t = '', a = '0123456789abcdef', n = 0; n < r.length; n++) {
        var o = r.charCodeAt(n);
        t += a.charAt((o >>> 4) & 15) + a.charAt(15 & o);
    }
    return t;
}
function _btr(r) {
    for (var t = '', a = 0, n = 32 * r.length; a < n; a += 8)
        t += String.fromCharCode((r[a >> 5] >>> (24 - (a % 32))) & 255);
    return t;
}
function _rtb(r) {
    for (var t = new Array(r.length >> 2), a = 0, n = t.length; a < n; a++)
        t[a] = 0;
    for (a = 0, n = 8 * r.length; a < n; a += 8)
        t[a >> 5] |= (255 & r.charCodeAt(a / 8)) << (24 - (a % 32));
    return t;
}
function _stb(r, t) {
    (r[t >> 5] |= 128 << (24 - (t % 32))), (r[15 + (((t + 64) >> 9) << 4)] = t);
    for (var a = new Array(80), n = 1732584193, o = -271733879, e = -1732584194, d = 271733878, f = -1009589776, h = 0, i = r.length; h < i; h += 16) {
        for (var u = n, _ = o, c = e, g = d, C = f, v = 0; v < 80; v++) {
            a[v] = v < 16 ? r[h + v] : _rl(a[v - 3] ^ a[v - 8] ^ a[v - 14] ^ a[v - 16], 1);
            var w = _ad(_ad(_rl(n, 5), _ft(v, o, e, d)), _ad(_ad(f, a[v]), _kt(v)));
            (f = d), (d = e), (e = _rl(o, 30)), (o = n), (n = w);
        }
        (n = _ad(n, u)), (o = _ad(o, _)), (e = _ad(e, c)), (d = _ad(d, g)), (f = _ad(f, C));
    }
    return [n, o, e, d, f];
}
function _sr(r) {
    return _btr(_stb(_rtb(r), 8 * r.length));
}
function _str(r) {
    for (var t, a, n = '', o = -1, e = r.length; ++o < e;)
        (t = r.charCodeAt(o)),
            (a = o + 1 < e ? r.charCodeAt(o + 1) : 0),
            55296 <= t &&
                t <= 56319 &&
                56320 <= a &&
                a <= 57343 &&
                ((t = 65536 + ((1023 & t) << 10) + (1023 & a)), ++o),
            t <= 127
                ? (n += String.fromCharCode(t))
                : t <= 2047
                    ? (n += String.fromCharCode(192 | ((t >>> 6) & 31), 128 | (63 & t)))
                    : t <= 65535
                        ? (n += String.fromCharCode(224 | ((t >>> 12) & 15), 128 | ((t >>> 6) & 63), 128 | (63 & t)))
                        : t <= 2097151 &&
                            (n += String.fromCharCode(240 | ((t >>> 18) & 7), 128 | ((t >>> 12) & 63), 128 | ((t >>> 6) & 63), 128 | (63 & t)));
    return n;
}
/* eslint-enable */
/**
 * Calculates SHA1 for the specified message
 * @param message {String} Message to calculate SHA1 for
 * @returns {String} Calculated SHA1
 */
export function sha1(message) {
    return _rth(_sr(_str(message)));
}
/**
 * Wraps specified asynchronous method with timing out logic.
 *
 * @param fn Asynchronous method to wrap
 * @param timeout Optinal timeout in milliseconds. Valid values are greater than 0. Anything else will be ignored
 * @returns
 */
export function wrapAsyncTimeout(fn, timeout) {
    if (!fn || !isFunction(fn) || !timeout) {
        return fn;
    }
    if (timeout < 0) {
        return fn;
    }
    return async function wrapAsyncInner(...args) {
        // eslint-disable-next-line no-async-promise-executor
        return new Promise(async (resolve, reject) => {
            let timedOut = false;
            const timeoutHandle = setTimeout(() => {
                timedOut = true;
                reject(new Error('Awaited asynchronous call has timed out'));
            }, timeout);
            let result;
            try {
                result = await fn(...args);
            }
            catch (e) {
                if (!timedOut) {
                    clearTimeout(timeoutHandle);
                    reject(e);
                    return;
                }
            }
            if (!timedOut) {
                clearTimeout(timeoutHandle);
                resolve(result);
            }
        });
    };
}
export function autoParseJson(value) {
    if (isObject(value) || isNil(value)) {
        return value;
    }
    try {
        return JSON.parse(value);
    }
    catch (e) {
        return null;
    }
}
export function getGlobalObject() {
    /* eslint-disable no-nested-ternary */
    return (typeof global !== 'undefined'
        ? global
        : typeof window !== 'undefined' // eslint-disable-line no-restricted-globals
            ? window // eslint-disable-line no-restricted-globals
            : typeof self !== 'undefined'
                ? self
                : {});
    /* eslint-enable no-nested-ternary */
}
function isValidHexColor(hexColor) {
    if (hexColor.length !== 8 &&
        hexColor.length !== 6 &&
        hexColor.length !== 4 &&
        hexColor.length !== 3) {
        return false;
    }
    return hexRgx.test(hexColor);
}
export function hexColorToRgba(hexColor) {
    // [R, G, B, A] with pre-set alpha (if it's not in the hex string)
    const components = [0, 0, 0, 255];
    // strip number sign
    // eslint-disable-next-line no-param-reassign
    hexColor = hexColor.replace('#', '');
    if (!isValidHexColor(hexColor)) {
        return null;
    }
    // get component size (the size or R, G, B and A within the string)
    const componentSize = hexColor.length < 6 ? 1 : 2;
    // componentIndex is used for convenience and speed,
    // but can be replaced with something like
    // (index / componentSize) >>> 0
    for (let index = componentSize - 1, componentIndex = 0; index < hexColor.length; index += componentSize, componentIndex += 1) {
        const element = hexColor[index] + hexColor[index - (componentSize - 1)];
        components[componentIndex] = parseInt(element, 16);
    }
    return components;
}
export function markObjectAsPatched(obj) {
    Object.defineProperty(obj, 'bugseePatched', {
        enumerable: false,
        configurable: false,
        writable: false,
        value: true,
    });
    return obj;
}
