import { NativeModules, Platform } from 'react-native';
import { getGlobalObject } from '../../common/utils';
import { getBuildId } from '../../version';
import { checkPromiseVersion, patchGlobalPromise, patchRegejectionTracking } from './patches';
import { parseAndSymbolicateErrorStack } from './stack-trace';
import { calculateSignature, cleanAndConstructBugseeExceptionFrame, getErrorName, isAggregateError, makeValidError, } from './utils';
const BugseeBridge = NativeModules.Bugsee;
const platformOS = Platform.OS;
/**
 *
 * @param state
 * @param eventCallback
 */
export function setupExceptionHandler(state, eventCallback) {
    const $global = getGlobalObject();
    const existingHandler = $global.ErrorUtils.getGlobalHandler && $global.ErrorUtils.getGlobalHandler();
    const onError = async (error, isFatal) => {
        if (state.launched && state.enabled) {
            try {
                await eventCallback(error, false);
            }
            catch (e) {
                // ignore
            }
        }
        if (existingHandler) {
            if (Platform.OS === 'ios') {
                existingHandler(error, isFatal);
            }
            else {
                setTimeout(() => existingHandler(error, isFatal), 500);
            }
        }
        return Promise.resolve();
    };
    $global.ErrorUtils.setGlobalHandler(onError);
}
/**
 *
 * @param state
 * @param eventCallback
 */
export function setupRejectionHandler(state, eventCallback) {
    patchGlobalPromise();
    patchRegejectionTracking(state, eventCallback);
    try {
        // eslint-disable-next-line global-require, import/no-extraneous-dependencies, @typescript-eslint/no-var-requires, import/no-unresolved
        const tracker = require('promise/setimmediate/rejection-tracking');
        // tracker.disable();
        tracker.enable({
            allRejections: true,
            onUnhandled: (id, error) => {
                if (state.launched && state.enabled) {
                    if (state.options?.handleRejections) {
                        eventCallback(error, true);
                    }
                }
            },
            onHandled: () => {
                // NOOP
            },
        });
    }
    catch (e) {
        // ignores
    }
    checkPromiseVersion();
}
async function createBugseeExceptionBase(error) {
    const name = getErrorName(error);
    const reason = error.message?.trim() || '';
    const { stack } = await parseAndSymbolicateErrorStack(error);
    if (error.generated) {
        // when stack trace is generated, try to remove all
        // extra frames
        const logFrameIndex = stack.findIndex((frame) => frame.methodName && frame.methodName.indexOf('Bugsee#logException') > -1);
        if (logFrameIndex > -1 && logFrameIndex < stack.length - 1) {
            stack.splice(0, logFrameIndex + 1);
        }
    }
    const frames = stack.map(cleanAndConstructBugseeExceptionFrame);
    const result = {
        name,
        reason,
        frames,
    };
    // check whether we have a nested error, and if we do,
    // handle it too
    if (error.cause) {
        result.cause = await createBugseeExceptionBase(error.cause);
    }
    return result;
}
async function createBugseeException(error) {
    const exceptionBase = await createBugseeExceptionBase(error);
    const signature = calculateSignature(exceptionBase);
    return {
        ...exceptionBase,
        signature,
        build_id: getBuildId(),
        platform_os: platformOS,
    };
}
async function logSingleException(exception, isHandled, options) {
    const error = makeValidError(exception);
    const bugseeException = await createBugseeException(error);
    const exceptionJson = JSON.stringify(bugseeException);
    // Log BugseeException
    await BugseeBridge.logException(exceptionJson, isHandled || false, options || null);
}
export async function logExceptionInternal(exception, isHandled, options) {
    if (isAggregateError(exception) && exception?.errors) {
        // in case of an AggregateError, we handle each Error
        // instance inside as a standalone error
        await Promise.all(exception.errors.map((errorItem) => logSingleException(errorItem, isHandled, options)));
    }
    else {
        await logSingleException(exception, isHandled, options);
    }
}
export function logExceptionInternalSync(exception, isHandled, options) {
    // this is required here to let us reconstruct the
    // stack trace in case of a invalid error (non-Error
    // or missing 'stack'). Also, making this method
    // async is not desired for 'fake' errors, as in this
    // case stack trace will be filled with intermediate
    // frames from regenerator-runtime which is required
    // by React Native itself
    const error = makeValidError(exception);
    logExceptionInternal(error, isHandled, options);
}
