import { EXCEPTION_SIGNATURE_CLEANUP_REGEXPRS, GLOBAL_ERROR_TYPES_NAMES, } from '../../common/constants';
import { getGlobalObject, isNil, jsonStringify, sha1 } from '../../common/utils';
import { getBuildId } from '../../version';
/**
 *
 * @param exception Error instance to get name for
 * @returns
 */
export function getErrorName(exception) {
    const $global = getGlobalObject();
    for (let i = 0; i < GLOBAL_ERROR_TYPES_NAMES.length; i += 1) {
        const errName = GLOBAL_ERROR_TYPES_NAMES[i];
        if ($global[errName] && exception instanceof $global[errName]) {
            return errName;
        }
    }
    if (exception instanceof Function || exception instanceof Error) {
        return exception.name || '<Function>';
    }
    return exception.constructor?.name || '<UnknownError>';
}
/**
 * Extracts clean string which then can be used in exception signature
 * calculation.
 *
 * @param frame Frame to extract clean string for signature from
 * @returns
 */
function cleanFrameTraceForSignature(frame) {
    return EXCEPTION_SIGNATURE_CLEANUP_REGEXPRS.reduce((result, regex) => result.replace(regex, ''), frame.trace);
}
function concatExceptionsRecursiveForSingature(exceptionBase) {
    if (exceptionBase) {
        return (exceptionBase.name +
            exceptionBase.frames.map(cleanFrameTraceForSignature).join('') +
            concatExceptionsRecursiveForSingature(exceptionBase.cause));
    }
    return '';
}
/**
 * Calculates signature for the specified error name and
 * stack trace
 *
 * @param exceptionBase Basic exception information
 * @returns Exception signature
 */
export function calculateSignature(exceptionBase) {
    const amalgamatedExceptions = concatExceptionsRecursiveForSingature(exceptionBase);
    return sha1(`${amalgamatedExceptions}${getBuildId()}`);
}
/**
 * Creates {@link BugseeExceptionFrame} from the provided {@link ReactNativeFrame}
 * cleaning the former in the proper way
 *
 * @param frame Frame to extract and clean data from
 * @returns Array of {@link BugseeExceptionFrame}
 */
export function cleanAndConstructBugseeExceptionFrame(frame) {
    let { methodName: member, file: source, lineNumber: line, column } = frame;
    member = member || '<unknown>';
    line = line || null;
    column = column || null;
    if (member) {
        const extraDataIdx = member.indexOf('(address at');
        if (extraDataIdx >= 0) {
            member = member.substring(0, extraDataIdx).trim();
        }
    }
    source =
        source
            ?.replace(/^file:\/\//, '')
            .replace(/^address at /, '')
            .replace(/^.*\/[^.]+(\.app|CodePush|.*(?=\/))/, '') || '';
    const data = {
        member,
        source,
        line,
        column,
    };
    let trace = `${member} () (${source}`;
    trace += !isNil(line) ? `:${line}` : '';
    trace += !isNil(column) ? `:${column}` : '';
    trace += ')';
    const result = {
        traceRaw: frame.trace,
        trace,
        data,
    };
    if (frame.file && frame.trace) {
        result.user =
            !!column &&
                !!line &&
                !frame.file.includes('node_modules') &&
                !frame.file.includes('native code') &&
                !frame.trace.includes('(native)');
    }
    return result;
}
export function generateError(message) {
    let generatedError = null;
    try {
        throw new Error(message || '');
    }
    catch (e) {
        if (e instanceof Error) {
            generatedError = e;
        }
    }
    // TODO: How so? What should we do?
    generatedError = generatedError || new Error(message || '');
    generatedError.generated = true;
    return generatedError;
}
function stripStackPrefixes(error, prefixes) {
    let stack = (error.stack || '').trim();
    if (stack) {
        for (let prefixIndex = 0; prefixIndex < prefixes.length; prefixIndex += 1) {
            const prefix = (prefixes[prefixIndex] || '').trim();
            if (prefix) {
                const index = stack.indexOf(prefix);
                if (index === 0) {
                    stack = stack.substring(prefix.length).trim();
                }
            }
        }
    }
    // remove any leading and trailing whitespaces
    return stack.trim();
}
export function getCleanStack(error) {
    let stack = error.stack || '';
    // stack trace may contain an error name with reason in the first
    // lines
    if (stack) {
        stack = stripStackPrefixes(error, [
            error.name,
            // eslint-disable-next-line no-proto
            error.__proto__?.name,
            error.constructor?.name,
            'Error',
            ':',
            '\n',
            error.message,
        ]);
        if (stack.length > 0 && stack[0] === '\n') {
            stack = stack.substring(1);
        }
    }
    // Generic fallback
    return stack;
}
export function makeValidError(value) {
    if (value instanceof Error) {
        return value;
    }
    // TODO: Make sure the assumption below does not break the consequent processing
    if (value?.stack) {
        // we assume that presence of 'stack' property should
        // indicate valid error instance
        return value;
    }
    let message = '';
    if (typeof value === 'string') {
        message = value;
    }
    else {
        try {
            message = jsonStringify(value);
        }
        catch (e) {
            // ignore
        }
    }
    return generateError(message);
}
export function isAggregateError(error) {
    if (typeof AggregateError !== 'undefined') {
        return error instanceof AggregateError;
    }
    return false;
}
