import * as React from 'react';
import logger from './common/logger';
import { logExceptionInternalSync } from './components/exceptions/handlers';
function is17Plus(version) {
    const majorVersion = version.match(/^([^.]+)/);
    return majorVersion !== null && parseInt(majorVersion[0], 10) >= 17;
}
function addComponentStackToError(error, componentStack) {
    if (is17Plus(React.version)) {
        const nestedError = new Error(error.message);
        nestedError.name = `ErrorBoundary ${nestedError.name}`;
        nestedError.stack = componentStack || '';
        // eslint-disable-next-line no-param-reassign
        error.cause = nestedError;
    }
}
const DEFAULT_STATE = {
    componentStack: null,
    error: null,
};
/**
 * An ErrorBoundary component that intercepts errors/exceptions in nested
 * React Native components and automatically sends them to Bugsee
 */
class ErrorBoundary extends React.Component {
    constructor(props) {
        super(props);
        this.state = DEFAULT_STATE;
    }
    componentDidCatch(error, { componentStack }) {
        const { onError } = this.props;
        addComponentStackToError(error, componentStack);
        logExceptionInternalSync(error, true);
        onError?.(error, componentStack);
        this.setState({ error, componentStack });
    }
    resetErrorBoundary = () => {
        const { onReset } = this.props;
        const { error, componentStack } = this.state;
        onReset?.(error, componentStack);
        this.setState(DEFAULT_STATE);
    };
    render() {
        const { fallback, children } = this.props;
        const { error, componentStack } = this.state;
        if (error) {
            const element = typeof fallback === 'function'
                ? fallback({ error, componentStack, resetErrorBoundary: this.resetErrorBoundary })
                : fallback;
            if (React.isValidElement(element)) {
                return element;
            }
            if (fallback && typeof __DEV__ !== 'undefined' && !!__DEV__) {
                logger.warn('ErrorBoundary fallback produced invalid ReactElement');
            }
            // Fail if no fallback is provided
            return null;
        }
        return typeof children === 'function' ? children() : children;
    }
}
ErrorBoundary.defaultProps = {
    children: null,
    fallback: null,
    onError: null,
    onReset: null
};
export default ErrorBoundary;
