/* eslint-disable class-methods-use-this */
import { NativeModules, findNodeHandle } from 'react-native';
import { getBuild, getVersion, setAppVersion } from './version';
import { AndroidLaunchOptions, IOSLaunchOptions, BugseeLaunchOptions, createDefaultLaunchOptions, } from './options';
import { autoParseJson } from './common/utils';
import Appearance from './appearance';
import { intializeComponents, setComponentsLaunched, setComponentsOptions } from './components';
import consoleComponent from './components/console';
import attachmentsComponent from './components/attachments';
import feedbackComponent from './components/feedback';
import lifecycleComponent from './components/lifecycle';
import networkComponent from './components/network';
import exceptionsComponent from './components/exceptions';
import ErrorBoundary from './errorboundary';
const BugseeBridge = NativeModules.Bugsee;
async function assembleLaunchOptions(newLaunchOptions) {
    const launchOptionsJson = await BugseeBridge.getLaunchOptions();
    return {
        ...BugseeLaunchOptions.serialize(createDefaultLaunchOptions(), true),
        ...(autoParseJson(launchOptionsJson) || {}),
        ...(newLaunchOptions || {}),
    };
}
class Bugsee {
    $lastOptions;
    $appearance;
    constructor() {
        this.$lastOptions = null;
        this.$appearance = new Appearance();
    }
    setLaunchOptions(launchOptions) {
        this.$lastOptions = {
            ...BugseeLaunchOptions.serialize(launchOptions, true),
        };
        setComponentsOptions(this.$lastOptions);
    }
    /**
     * Initializes Bugsee instance and triggers installation of
     * various hooks. Must be called as early as possible.
     */
    async launch(appToken, launchOptions) {
        await setAppVersion(null);
        const assembledLaunchOptions = await assembleLaunchOptions(BugseeLaunchOptions.serialize(launchOptions, true));
        const isLaunched = await BugseeBridge.launch(appToken, assembledLaunchOptions);
        if (isLaunched) {
            intializeComponents(assembledLaunchOptions);
            this.setLaunchOptions(assembledLaunchOptions);
            setComponentsLaunched(true);
        }
    }
    /**
     * Restarts already launched session with the new set of options
     */
    async relaunch(launchOptions) {
        // TODO: check whether Bugsee is launched!!!
        const assembledLaunchOptions = await assembleLaunchOptions(BugseeLaunchOptions.serialize(launchOptions, true));
        const isRelaunched = await BugseeBridge.relaunch(assembledLaunchOptions);
        if (isRelaunched) {
            this.setLaunchOptions(assembledLaunchOptions);
            setComponentsLaunched(true);
        }
    }
    /**
     * Initializes wrapping layer assuming native layer is already
     * launched and running
     */
    async initialize() {
        await setAppVersion(null);
        const assembledLaunchOptions = await assembleLaunchOptions({});
        intializeComponents(assembledLaunchOptions);
        this.setLaunchOptions(assembledLaunchOptions);
        setComponentsLaunched(true);
    }
    /**
     * Stops currently active (running) session
     */
    async stop() {
        const isStopped = await BugseeBridge.stop();
        if (isStopped) {
            setComponentsLaunched(false);
        }
    }
    /**
     * Displays Bugsee's default reporting UI
     */
    showReportDialog(summary, description, severity) {
        BugseeBridge.showReportDialog({ summary, description, severity });
    }
    /**
     * Initiates bug report upload without displaying any UI
     *
     * @param summary Short description for the bug report. Required
     * @param description Optional long description
     * @param severity Optional severity of the bug report
     * @param labels Optional labels to associate with the bug report
     * @param includeVideo Flag, which if set to false will instruct the underlying SDK to exclude video from the final bundle
     */
    upload(summary, description, severity, labels, includeVideo = true) {
        BugseeBridge.upload({
            summary,
            description,
            severity,
            labels,
            includeVideo,
        });
    }
    /**
     * Adds an event with specified name and additional parameters
     */
    event(name, params) {
        BugseeBridge.event({ name, params });
    }
    /**
     * Adds new value for the specified trace
     */
    trace(name, value) {
        BugseeBridge.trace({ name, value });
    }
    /**
     * Assigns specified email to the application & device pair. It will be
     * persisted until either reset with {@link clearEmail} or application
     * is deleted. This email will be attached to all following issue reports.
     * @param email
     */
    setEmail(email) {
        BugseeBridge.setEmail({ email });
    }
    /**
     * Gets email
     */
    async getEmail() {
        // eslint-disable-next-line no-return-await
        return await BugseeBridge.getEmail();
    }
    /**
     * Removes previously set with {@link setEmail} value.
     */
    clearEmail() {
        BugseeBridge.clearEmail();
    }
    /**
     * Sets persistent attribute value.
     * @param name Attribute name
     * @param value Attribute value
     */
    setAttribute(name, value) {
        BugseeBridge.setAttribute({ name, value });
    }
    /**
     * Gets persistent attribute value
     * @param name Name of the attribute to get value for
     * @returns
     */
    async getAttribute(name) {
        // eslint-disable-next-line no-return-await
        return await BugseeBridge.getAttribute(name);
    }
    /**
     * Removes previously set attribute
     * @param name Attribute name
     */
    clearAttribute(name) {
        BugseeBridge.clearAttribute({ name });
    }
    /**
     * Removes all persistent attributes
     */
    clearAllAttributes() {
        BugseeBridge.clearAllAttributes();
    }
    /**
     * Pause video recording
     */
    pause() {
        BugseeBridge.pause();
    }
    /**
     * Resume video recording
     */
    resume() {
        BugseeBridge.resume();
    }
    /**
     * Toggle protected state for the specified component
     */
    toggleProtected(reactComponent, isProtected) {
        if (reactComponent) {
            // Call toggleProtected() method with a slight delay
            setTimeout(() => {
                try {
                    // this may throw if component is unmounted
                    const foundNode = findNodeHandle(reactComponent);
                    if (foundNode) {
                        BugseeBridge.toggleProtected(foundNode, isProtected);
                    }
                }
                catch (e) {
                    // ignore
                }
            }, 10);
        }
    }
    /**
     * Log specified message to Bugsee logs
     * @param {String} text
     * @param {LogLevel} level
     */
    log(text, level) {
        BugseeBridge.log({ text, level });
    }
    /**
     * Display Feedback UI
     */
    showFeedbackUI() {
        BugseeBridge.showFeedbackUI();
    }
    /**
     * Set default greeting that will be displayed when there are no
     * messages in Feedback
     */
    setDefaultFeedbackGreeting(message) {
        BugseeBridge.setDefaultFeedbackGreeting(message);
    }
    setNewFeedbackMessageCallback(callback) {
        feedbackComponent.setHandler(callback);
    }
    /**
     * Logs caught exception and reports it as an error to Bugsee servers
     * @param {Error} exception
     * @param {BugseeExceptionLoggingOptions} [options]
     */
    logException(exception, options) {
        exceptionsComponent.executeDefaultAction(exception, { options });
    }
    /**
     * Explicitly sets application version to use when sending
     * crash/error reports. If not set, it will be automatically
     * picked from application info.
     *
     * NOTE: To properly decipher minified stack traces, this
     * version must match the one specified when uploading
     * sourcemaps.
     * @param {String} appVersion New application version value
     * @param {String} appBuild New application version value
     */
    setAppVersion(appVersion, appBuild) {
        setAppVersion(appVersion, appBuild);
    }
    getAppVersion() {
        return getVersion();
    }
    getAppBuild() {
        return getBuild();
    }
    addSecureRect(x, y, width, height) {
        BugseeBridge.addSecureRect(x, y, width, height);
    }
    removeSecureRect(x, y, width, height) {
        BugseeBridge.removeSecureRect(x, y, width, height);
    }
    async getAllSecureRects() {
        const rawRects = await BugseeBridge.getAllSecureRects();
        return rawRects.map((rawRect) => {
            const [x, y, width, height] = rawRect;
            return { x, y, width, height };
        });
    }
    removeAllSecureRects() {
        BugseeBridge.removeAllSecureRects();
    }
    /**
     * Sets callback that will be called to sanitize
     * captured console event
     */
    setLogFilter(filterCallback) {
        // TODO: Full LogEvent should not be exposed
        consoleComponent.setHandler(filterCallback);
    }
    /**
     * Sets callback that will be called to sanitize
     * captured network event
     */
    setNetworkFilter(filterCallback) {
        // TODO: Full NetEvent should not be exposed
        networkComponent.setHandler(filterCallback);
    }
    setLifecycleEventHandler(lifecycleCallback) {
        lifecycleComponent.setHandler(lifecycleCallback);
    }
    /**
     * Sets the flag that controls visibility of the keyboard
     * in captured video
     */
    setKeyboardVisibility(isVisible) {
        BugseeBridge.hideKeyboard(!isVisible);
    }
    /**
     * Captures current view hierarchy
     */
    captureViewHierarchy() {
        BugseeBridge.captureViewHierarchy();
    }
    /**
     * Sets the handler which is executed when report is being
     * bundled and attachments are requested to be added to the
     * final bundle
     */
    setAttachmentsHandler(handler) {
        attachmentsComponent.setHandler(handler);
    }
    /**
     * Deletes all the collected data stored in the files on the disk. Recording must
     * be stopped via {@link #stop} before calling this method.
     *
     * @returns {Promise<boolean>} true if data was successfully deleted, false otherwise
     */
    async deleteCollectedDataOnDevice() {
        return Promise.race([
            BugseeBridge.deleteCollectedDataOnDevice(),
            new Promise((resolve) => {
                setTimeout(() => {
                    resolve(false);
                }, 5000);
            }),
        ]);
    }
    testNativeCrash() {
        BugseeBridge.testCrash();
    }
    testJsCrash() {
        throw new Error('Test error');
    }
    LogLevel = Object.freeze({
        None: 0,
        Error: 1,
        Warning: 2,
        Info: 3,
        Debug: 4,
        Verbose: 5,
    });
    Severity = Object.freeze({
        Low: 1,
        Medium: 2,
        High: 3,
        Critical: 4,
        Blocker: 5,
    });
    VideoMode = Object.freeze({
        None: 0,
        V1: 1,
        V2: 2,
        V3: 3,
    });
    VideoQuality = Object.freeze({
        Default: 0,
        Medium: 1,
        High: 2,
    });
    LifecycleEventsTypes = Object.freeze({
        /**
         * Event is dispatched when Bugsee was successfully launched
         */
        Launched: 0,
        /**
         * Event is dispatched when Bugsee is started after being stopped
         */
        Started: 1,
        /**
         * Event is dispatched when Bugsee is stopped
         */
        Stopped: 2,
        /**
         * Event is dispatched when Bugsee recording is resumed after being paused
         */
        Resumed: 3,
        /**
         * Event is dispatched when Bugsee recording is paused
         */
        Paused: 4,
        /**
         * Event is dispatched when Bugsee is launched and pending crash report is
         * discovered. That usually means that app was relaunched after crash.
         */
        RelaunchedAfterCrash: 5,
        /**
         * Event is dispatched before the reporting UI is shown
         */
        BeforeReportShown: 6,
        /**
         * Event is dispatched when reporting UI is shown
         */
        AfterReportShown: 7,
        /**
         * Event is dispatched when report is about to be uploaded to the server
         */
        BeforeReportUploaded: 8,
        /**
         * Event is dispatched when report was successfully uploaded to the server
         */
        AfterReportUploaded: 9,
        /**
         * Event is dispatched before the Feedback controller is shown
         */
        BeforeFeedbackShown: 10,
        /**
         * Event is dispatched after the Feedback controller is shown
         */
        AfterFeedbackShown: 11,
    });
    AndroidLaunchOptions = AndroidLaunchOptions;
    IOSLaunchOptions = IOSLaunchOptions;
    /**
     * ErrorBoundary which can be used to catch layout errors
     */
    ErrorBoundary = ErrorBoundary;
    /**
     * Appearance settings for Bugsee UI
     */
    get appearance() {
        return this.$appearance;
    }
}
const bugseeInstance = new Bugsee();
export default bugseeInstance;
